# -*- coding: utf-8 -*-

# GraphicsDialog.py
# This file is part of the Gedit LaTeX Plugin
#
# Copyright (C) 2006 Michael Zeising
#
# This program is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public Licence as published by the Free Software
# Foundation; either version 2 of the Licence, or (at your option) any later
# version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU General Public Licence for more 
# details.
#
# You should have received a copy of the GNU General Public Licence along with
# this program; if not, write to the Free Software Foundation, Inc., 51 Franklin
# Street, Fifth Floor, Boston, MA  02110-1301, USA

import gtk
from gettext import gettext as _
import const

U_PERCENT = 0
U_PX = 1
U_IN = 2
U_MM = 3

UNITS = {
		U_PERCENT : _("Percent"), 
		U_PX : _("Pixel"),
		U_IN : _("Inches"), 
		U_MM : _("Millimeter")
}

class GraphicsDialog(gtk.Dialog):
	_source = None
	_aspect = 0.0		# original aspect ratio of the image
	_locked = True		# keep aspect ratio
	_last_unit = 0
	
	def __init__(self, dpi):
		self.dpi = dpi
		gtk.Dialog.__init__(self, title=_("Include Graphics"), 
				flags=gtk.DIALOG_DESTROY_WITH_PARENT, 
				buttons=(gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL, gtk.STOCK_OK, gtk.RESPONSE_OK))
		self.set_border_width(5)
		self.set_has_separator(False)
		self.vbox.set_spacing(10)
		
		###########
		# file
		self.fileDialog = gtk.FileChooserDialog(parent=self, action=gtk.FILE_CHOOSER_ACTION_OPEN,
				buttons=(gtk.STOCK_OK, gtk.RESPONSE_ACCEPT, gtk.STOCK_CANCEL, gtk.RESPONSE_REJECT))
		imgPreview = gtk.Image()
		self.fileDialog.set_preview_widget(imgPreview)
		self.fileDialog.set_local_only(True)
		self.fileDialog.connect("update-preview", self.onUpdatePreview, imgPreview)
		self.fileDialog.connect("response", self.onFileResponse)
		self.fileButton = gtk.FileChooserButton(self.fileDialog)
		
		algFile = gtk.Alignment(.5, .5, 1, 1)
		algFile.set_padding(5, 0, 15, 0)
		algFile.add(self.fileButton)
		
		frmFile = gtk.Frame(_("<b>_File</b>"))
		frmFile.get_label_widget().set_use_markup(True)
		frmFile.get_label_widget().set_use_underline(True)
		frmFile.get_label_widget().set_mnemonic_widget(self.fileButton)
		frmFile.set_shadow_type(gtk.SHADOW_NONE)
		frmFile.add(algFile)
		
		###########
		# image size
		self.spinX = gtk.SpinButton()
		self.spinX.set_range(1, 16000)
		self.spinX.set_numeric(True)
		self.spinX.set_digits(2)
		self.spinX.set_increments(1, 10)
		
		self.spinY = gtk.SpinButton()
		self.spinY.set_range(1, 16000)
		self.spinY.set_numeric(True)
		self.spinY.set_digits(2)
		self.spinY.set_increments(1, 10)
		
		lblWidth = gtk.Label(_("_Width:"))
		lblWidth.set_use_underline(True)
		lblWidth.set_mnemonic_widget(self.spinX)
		lblWidth.set_alignment(0, .5)
		lblHeight = gtk.Label(_("_Height:"))
		lblHeight.set_use_underline(True)
		lblHeight.set_mnemonic_widget(self.spinY)
		lblHeight.set_alignment(0, .5)
		
		self.tglLock = gtk.ToggleButton(None)
		self.tglLock.set_relief(gtk.RELIEF_NONE)
		image = gtk.Image()
		image.set_from_file(const.PATH + "/pixmaps/lock.png")
		self.tglLock.add(image)
		self.tglLock.set_active(True)
		
		self.cmbUnit = gtk.combo_box_new_text()
		for key in UNITS:
			self.cmbUnit.append_text(UNITS[key])
		self.cmbUnit.set_active(0)
		
		self.lblOrigSize = gtk.Label()
		self.lblOrigSize.set_alignment(0, .5)
		
		tblSize = gtk.Table()
		tblSize.set_row_spacings(5)
		tblSize.set_col_spacings(5)
		tblSize.attach(lblWidth, 0, 1, 0, 1)
		tblSize.attach(lblHeight, 0, 1, 1, 2)
		tblSize.attach(self.lblOrigSize, 1, 4, 2, 3)
		tblSize.attach(self.spinX, 1, 2, 0, 1, False, False)
		tblSize.attach(self.spinY, 1, 2, 1, 2, False, False)
		tblSize.attach(self.tglLock, 2, 3, 0, 2, False)
		tblSize.attach(self.cmbUnit, 3, 4, 1, 2, False, False)
		
		algSize = gtk.Alignment(.5, .5, 1, 1)
		algSize.set_padding(5, 0, 15, 0)
		algSize.add(tblSize)
		
		self.frmSize = gtk.Frame(_("<b>Size</b>"))
		self.frmSize.get_label_widget().set_use_markup(True)
		self.frmSize.set_shadow_type(gtk.SHADOW_NONE)
		self.frmSize.add(algSize)
		
		###########
		# position
		#
		# b - bottom of page
		# t - top of page
		# h - here
		self.cmbPos = gtk.combo_box_new_text()
		for item in [_("Here"), _("Top of page"), _("Bottom of page")]:
			self.cmbPos.append_text(item)
		self.cmbPos.set_active(0)
		
		lblPos = gtk.Label(_("Desired _Position:"))
		lblPos.set_use_underline(True)
		lblPos.set_mnemonic_widget(self.cmbPos)
		lblPos.set_alignment(0, .5)
		
		hboxPos = gtk.HBox()
		hboxPos.set_spacing(5)
		hboxPos.pack_start(lblPos, False)
		hboxPos.pack_start(self.cmbPos, False)

		algPos = gtk.Alignment(.5, .5, 1, 1)
		algPos.set_padding(5, 0, 15, 0)
		algPos.add(hboxPos)
		
		self.frmPos = gtk.Frame(_("<b>Position</b>"))
		self.frmPos.get_label_widget().set_use_markup(True)
		self.frmPos.set_shadow_type(gtk.SHADOW_NONE)
		self.frmPos.add(algPos)
		
		###########
		# caption
		self.entryCaption = gtk.Entry()
		
		algCaption = gtk.Alignment(.5, .5, 1, 1)
		algCaption.set_padding(5, 0, 15, 0)
		algCaption.add(self.entryCaption)
		
		self.frmCaption = gtk.Frame(_("<b>_Caption</b>"))
		self.frmCaption.get_label_widget().set_use_underline(True)
		self.frmCaption.get_label_widget().set_use_markup(True)
		self.frmCaption.get_label_widget().set_mnemonic_widget(self.entryCaption)
		self.frmCaption.set_shadow_type(gtk.SHADOW_NONE)
		self.frmCaption.add(algCaption)
		
		box = gtk.VBox()
		box.set_border_width(5)
		box.set_spacing(5)
		box.pack_start(frmFile, False)
		box.pack_start(self.frmSize, False)
		box.pack_start(self.frmPos, False)
		box.add(self.frmCaption)
		
		self.vbox.add(box)
		
		self.connect("response", self.onResponse)
		self.fileDialog.connect("file-activated", self.onFileActivated)
		self.spinX.connect("value-changed", self.onSizeChanged, 0)
		self.spinY.connect("value-changed", self.onSizeChanged, 1)
		self.cmbUnit.connect("changed", self._on_unit_changed)
		self.set_default_response(gtk.RESPONSE_OK)
		
		self.set_resizable(False)
	
	def run(self):
		self.reset()
		self.show_all()
		#self.fileDialog.show()
		return gtk.Dialog.run(self)
	
	def reset(self):
		self.frmSize.set_sensitive(False)
		self.frmPos.set_sensitive(False)
		self.frmCaption.set_sensitive(False)
		self.spinX.set_value(1)
		self.spinY.set_value(1)
		self.cmbUnit.set_active(0)
		self.lblOrigSize.set_text("")
		self.cmbPos.set_active(0)
		self.entryCaption.set_text("")
	
	def onUpdatePreview(self, file_chooser, preview):
		"""
		Updates the FileChooser's preview image
		"""
		filename = file_chooser.get_preview_filename()
		try:
			pixbuf = gtk.gdk.pixbuf_new_from_file_at_size(filename, 128, 128)
			preview.set_from_pixbuf(pixbuf)
			have_preview = True
		except:
			have_preview = False
		file_chooser.set_preview_widget_active(have_preview)
	
	def onFileResponse(self, dialog, response):
		if response == gtk.RESPONSE_ACCEPT:
			self.onFileActivated(dialog)
	
	def onFileActivated(self, file_chooser):
		"""
		Updates the widgets according to the chosen file
		"""
		# reset ui
		self.cmbUnit.set_active(0)
		self._last_unit = 0
		self.spinX.set_value(100)
		self.spinY.set_value(100)
		
		try:
			# load the image to a pixbuf
			pixbuf = gtk.gdk.pixbuf_new_from_file(file_chooser.get_filename())
			# store original size and aspect ratio
			self._size = (pixbuf.get_width(), pixbuf.get_height())
			self._aspect = float(self._size[0]) / float(self._size[1])
			# show the image's original size
			self.lblOrigSize.set_markup("<small>%s x %s Pixel</small>" % (self._size[0], self._size[1]))
		except:
			pass
		self._activate()
	
	def _activate(self):
		"""
		Enables the UI after an image has been selected
		"""
		self.frmSize.set_sensitive(True)
		self.frmPos.set_sensitive(True)
		self.frmCaption.set_sensitive(True)
	
	def _on_unit_changed(self, combo_box):
		"""
		Convert the current size to the new unit
		"""
		x = float(self.spinX.get_value())
		y = float(self.spinY.get_value())
		# convert the current size to pixels
		if self._last_unit == U_PERCENT:
			pixels = ((x/100) * self._size[0], (y/100) * self._size[1])
		elif self._last_unit == U_PX:
			pixels = (x, y)
		elif self._last_unit == U_IN:
			pixels = (x * self.dpi, y * self.dpi)
		elif self._last_unit == U_MM:
			# 1 in = 25.4 mm
			pixels = ((x / 25.4) * self.dpi, (y / 25.4) * self.dpi)
		# convert to pixels to the new unit
		unit = combo_box.get_active()
		if unit == U_PERCENT:
			size = ((pixels[0] / self._size[0]) * 100, (pixels[1] / self._size[1]) * 100)
		elif unit == U_PX:
			size = pixels
		elif unit == U_IN:
			size = (pixels[0] / self.dpi, pixels[1] / self.dpi)
		elif unit == U_MM:
			size = ((pixels[0] / self.dpi) * 25.4, (pixels[1] / self.dpi) * 25.4)
		self._last_unit = unit
		# set the spin buttons
		self.spinX.set_value(size[0])
		self.spinY.set_value(size[1])
		# set digits
		if unit == U_PX:
			self.spinX.set_digits(0)
			self.spinY.set_digits(0)
		else:
			self.spinX.set_digits(2)
			self.spinY.set_digits(2)
	
	def onSizeChanged(self, spin, axis):
		"""
		Adjusts the other axis, if the aspect ratio is to be kept
		"""
		if not self.tglLock.get_active():
			return
		if self.cmbUnit.get_active() == U_PERCENT:
			if axis:
				# y changed
				self.spinX.set_value(spin.get_value())
			else:
				# x changed
				self.spinY.set_value(spin.get_value())
		elif self.cmbUnit.get_active() == U_PX:
			# pixel selected
			if axis:
				# y changed
				self.spinX.set_value(round(spin.get_value() * self._aspect))
			else:
				# x changed
				self.spinY.set_value(round(spin.get_value() / self._aspect))
		else:
			# metric unit selected
			if axis:
				# y changed
				self.spinX.set_value(spin.get_value() * self._aspect)
			else:
				# x changed
				self.spinY.set_value(spin.get_value() / self._aspect)
	
	def onResponse(self, dialog, response):
		if response == gtk.RESPONSE_OK:
			self._assembleSource()
	
	def _assembleSource(self):
		"""
		Assembles the LaTeX source
		"""
		filename = self.fileDialog.get_filename()
		unit = self.cmbUnit.get_active()
		x = self.spinX.get_value()
		y = self.spinY.get_value()
		caption = self.entryCaption.get_text()
		
		s = "\\includegraphics{%s}" % filename
		if unit == U_PERCENT:
			if x != 100 or y != 100:
				s = "\\scalebox{%s}[%s]{%s}" % ((x/100), (y/100), s)
		else:
			unit_s = ["","px","in","mm"][unit]
			s = "\\resizebox{%s%s}{%s%s}{%s}" % (x, unit_s, y, unit_s, s)
		pos = ["h","t","b"][self.cmbPos.get_active()]
		s = "\\begin{figure}[%s]\n\t%s" % (pos, s)
		if len(caption):
			s = "%s\n\t\\caption{%s}" % (s, caption)
		self.source = "%s\n\\end{figure}" % s
	
