# -*- coding: utf-8 -*-

# macros.py
# This file is part of the Gedit LaTeX Plugin
#
# Copyright (C) 2006 Michael Zeising
#
# This program is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public Licence as published by the Free Software
# Foundation; either version 2 of the Licence, or (at your option) any later
# version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU General Public Licence for more 
# details.
#
# You should have received a copy of the GNU General Public Licence along with
# this program; if not, write to the Free Software Foundation, Inc., 51 Franklin
# Street, Fifth Floor, Boston, MA  02110-1301, USA

import gtk, re
from gettext import gettext as _
from common import *
import const

# TODO:
# * \newcommand: context menu with "Go to definition", "Insert command" and "Delete"
# * user-defined macros should be stored under internal keys (maybe md5 of the definition) 
#   and their name should be stored in /name

# scopes for macros
S_BOTH = 0
S_GENERAL_ONLY = 1
S_MATH_ONLY = 2

# the built-in macros: (label, filename, markup, scope)
BUILTIN_MACROS = [
		(_("Root"), "root", "\\sqrt[n]{x}", S_MATH_ONLY),
		(_("Sum"), "sum", "\\sum\\limits_{k=1}^{N} k^2", S_MATH_ONLY),
		(_("Product"), "product", "\\prod\\limits_{i=1}^{N} x_i", S_MATH_ONLY),
		(_("Union"), "union", "\\bigcup\\limits_{\\lambda\\in\\Lambda} A_\\lambda", S_MATH_ONLY),
		(_("Intersection"), "intersection", "\\bigcap\\limits_{\\lambda\\in\\Lambda} A_\\lambda", S_MATH_ONLY),
		(_("Limes"), "limes", "\\lim\\limits_{n \\to \\infty}x_n", S_MATH_ONLY),
		(_("Integral"), "integral", "\\int\\limits_{-N}^{N} \\mathrm{e}^x\\, \\mathrm{d}x", S_MATH_ONLY),
		(_("Double sub/superscript"), "subsuper", "{}^1_2\\mathrm{C}^3_4", S_MATH_ONLY)
]

reArgRef = re.compile(r"#[1-9]")

class Macros(gtk.Notebook):
	_macroDialog = None
	
	def __init__(self, instance):
		self._instance = instance
		gtk.Notebook.__init__(self)
		
		self.appendMacrosPage()
		self.appendNewcommandsPage()
		
		self.show_all()
		
		self._instance._prefs.add_notify(lambda *a: self.updateUserMacros(), "/user-defined-macros")
	
	def appendMacrosPage(self):
		""" Appends a page to the notebook containing the built-in and user-defined macros """
		self.panedMacros = gtk.VPaned()
		
		# --- built-in macros ---
		storeBuiltin = gtk.ListStore(str, gtk.gdk.Pixbuf)
		
		for macro in BUILTIN_MACROS:
			pixbuf = gtk.gdk.pixbuf_new_from_file("%s/pixmaps/macros/%s.png" % (const.PATH, macro[1]))
			storeBuiltin.append([macro[0], pixbuf])
		
		viewBuiltin = gtk.IconView(storeBuiltin)
		viewBuiltin.set_pixbuf_column(1)
		viewBuiltin.set_selection_mode(gtk.SELECTION_SINGLE)
		viewBuiltin.connect("selection-changed", self.onBuiltinMacroSelected)
		viewBuiltin.set_spacing(3)
		viewBuiltin.set_columns(-1)			# show as much as possible
		viewBuiltin.set_text_column(0)
	
		scrBuiltin = gtk.ScrolledWindow()
		scrBuiltin.set_policy(gtk.POLICY_NEVER, gtk.POLICY_AUTOMATIC);
		scrBuiltin.set_shadow_type(gtk.SHADOW_IN)
		scrBuiltin.add(viewBuiltin)
		
		self.panedMacros.add1(scrBuiltin)
		
		# --- user defined macros ---
		vbox = gtk.VBox()
		
		# name, definition, scope, checksum, packages (,-sep)
		self.storeUser = gtk.ListStore(str, str, int, str, str, str)
		self.updateUserMacros()
		
		self.viewUser = gtk.TreeView(self.storeUser)
		self.viewUser.insert_column_with_attributes(-1, _("Name"), gtk.CellRendererText(), markup=5)
		self.viewUser.insert_column_with_attributes(-1, _("Definition"), gtk.CellRendererText(), text=1)
		self.viewUser.connect("row-activated", self.onUserMacroSelected)
		self.viewUser.connect("button-press-event", self.onUserMacroContext)
	
		scrUser = gtk.ScrolledWindow()
		scrUser.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
		scrUser.set_shadow_type(gtk.SHADOW_IN)
		scrUser.add(self.viewUser)
		
		vbox.pack_start(scrUser, True)
		
		btnAddUserMacro = gtk.Button(_("Add Macro"))
		btnAddUserMacro.set_relief(gtk.RELIEF_NONE)
		btnAddUserMacro.set_image(gtk.image_new_from_stock(gtk.STOCK_ADD, gtk.ICON_SIZE_MENU))
		btnAddUserMacro.connect("clicked", lambda a: self.runMacroDialog(const.ADD_MACRO))
		
		vbox.pack_start(btnAddUserMacro, False)
		
		self.panedMacros.add2(vbox)
		
		self.append_page(self.panedMacros, gtk.Label(_("Macros")))
	
	def runMacroDialog(self, mode, macroChecksum=None):
		if not self._macroDialog:
			self._macroDialog = MacroDialog(self._instance)
		if mode == const.EDIT_MACRO:
			name = self._instance._prefs["/user-defined-macros/%s/name" % macroChecksum]
			definition = self._instance._prefs["/user-defined-macros/%s/definition" % macroChecksum]
			scope = self._instance._prefs["/user-defined-macros/%s/scope" % macroChecksum]
			packages = self._instance._prefs.setdefault("/user-defined-macros/%s/packages" % macroChecksum, [])
			self._macroDialog.loadMacro(name, definition, scope, packages)
		if self._macroDialog.run(mode) == gtk.RESPONSE_OK:
			if mode == const.ADD_MACRO:
				import md5
				# add the macro
				checksum = md5.new(self._macroDialog.definition).hexdigest()
				subdir = "/user-defined-macros/" + checksum
				if self._instance._prefs.dir_exists(subdir):
					print "Error: A macro with that definition already exists."
				else:
					self._instance._prefs.add_dir(subdir)
					self._instance._prefs[subdir + "/name"] = self._macroDialog.macroName
					self._instance._prefs[subdir + "/scope"] = self._macroDialog.scope
					self._instance._prefs[subdir + "/definition"] = self._macroDialog.definition
					self._instance._prefs[subdir + "/packages"] = self._macroDialog.packages
			elif mode == const.EDIT_MACRO:
				# save changes to the macro
				subdir = "/user-defined-macros/" + macroChecksum
				self._instance._prefs[subdir + "/name"] = self._macroDialog.macroName
				self._instance._prefs[subdir + "/scope"] = self._macroDialog.scope
				self._instance._prefs[subdir + "/definition"] = self._macroDialog.definition
				self._instance._prefs[subdir + "/packages"] = self._macroDialog.packages
			elif mode == const.ADD_NEWCOMMAND:
				# add newcommand
				name = self._macroDialog.macroName
				if name[0] != "\\":
					name = "\\" + name
				try:
					refCount = max([match.group()[1] for match in reArgRef.finditer(self._macroDialog.definition)])
				except ValueError:
					refCount = 0
				self._instance.get_document().insert_newcommand(name, self._macroDialog.definition, refCount)
		self._macroDialog.hide()
	
	def updateUserMacros(self):
		""" Search the configuration for user-defined macros """
		self.storeUser.clear()
		for path in self._instance._prefs.get_subdirs("/user-defined-macros"):
			checksum = path.split("/")[-1]
			name = self._instance._prefs["/user-defined-macros/%s/name" % checksum]
			definition = self._instance._prefs["/user-defined-macros/%s/definition" % checksum]
			scope = self._instance._prefs["/user-defined-macros/%s/scope" % checksum]	# 0: both, 1: general only, 2: math only
			packages = ",".join(self._instance._prefs.setdefault("/user-defined-macros/%s/packages" % checksum, []))
			if name and definition:
				self.storeUser.append([name, definition, scope, checksum, packages, "<b>%s</b>" % name])
		
	def onBuiltinMacroSelected(self, widget):
		""" Inserts a built-in macro """
		try:
			index = widget.get_selected_items()[0][0]
		except IndexError:		# must be caught after unselect_all()
			return
		macro = BUILTIN_MACROS[index][2]
		scope = BUILTIN_MACROS[index][3]
		environment = self._instance.get_document().environment
		# consider the scope of the macro
		if (scope == S_GENERAL_ONLY and environment == E_MATH) or (scope == S_MATH_ONLY and environment != E_MATH):
			macro = "$ %s $" % macro
		# insert the macro
		self._instance.get_document().insert(macro)
		# reset selection
		widget.unselect_all()
	
	def onUserMacroSelected(self, treeview, path, column):	
		""" Inserts a user-defined macro """
		liststore = treeview.get_model()
		macro = liststore.get_value(liststore.get_iter(path), 1)
		scope = liststore.get_value(liststore.get_iter(path), 2)
		packages = liststore.get_value(liststore.get_iter(path), 4)
		self.insertUserMacro(macro, scope, packages)
	
	def onUserMacroContextInsert(self, action):
		""" Inserts a user-defined macro (called from the context menu) """
		liststore, listiter = self.viewUser.get_selection().get_selected()
		macro = liststore.get_value(listiter, 1)
		scope = liststore.get_value(listiter, 2)
		packages = liststore.get_value(listiter, 4)
		self.insertUserMacro(macro, scope, packages)
	
	def insertUserMacro(self, macro, scope, packages):
		environment = self._instance.get_document().environment
		# consider the scope of the macro
		if (scope == S_GENERAL_ONLY and environment == E_MATH) or (scope == S_MATH_ONLY and environment != E_MATH):
			macro = "$ %s $" % macro
		# insert the macro
		self._instance.get_document().insert(macro)
		# check required packages
		if len(packages):
			self._instance.get_document().requirePackages(packages.split(","))
	
	def onUserMacroContext(self, treeview, event):
		""" Displays the context menu for a user-defined macro """
		if event.button == 3:
			x, y = int(event.x), int(event.y)
			pathTuple = treeview.get_path_at_pos(x, y)
            
			if pathTuple is not None:
				path = pathTuple[0]
				treeview.grab_focus()
				treeview.set_cursor(path)
		
			selection = treeview.get_selection()
			if not selection.count_selected_rows():
				return
			self._instance._manager.get_widget("/MacroPopup").popup(None, None, None, event.button, event.time)
	
	def onUserMacroContextEdit(self, action):
		""" Shows the dialog for editing a user-defined macro """
		liststore, listiter = self.viewUser.get_selection().get_selected()
		checksum = liststore.get_value(listiter, 3)
		self.runMacroDialog(const.EDIT_MACRO, checksum)
		
	def onUserMacroContextDelete(self, action):
		"""
		Deletes a user-defined macro from the configuration
		"""
		liststore, listiter = self.viewUser.get_selection().get_selected()
		checksum = liststore.get_value(listiter, 3)
		# delete all entries of the macro
		self._instance._prefs.unset("/user-defined-macros/%s/name" % checksum)
		self._instance._prefs.unset("/user-defined-macros/%s/definition" % checksum)
		self._instance._prefs.unset("/user-defined-macros/%s/scope" % checksum)
		self._instance._prefs.unset("/user-defined-macros/%s/packages" % checksum)
	
	def appendNewcommandsPage(self):
		vbox = gtk.VBox()
		
		self.viewNewcommands = gtk.TreeView()
		self.viewNewcommands.insert_column_with_attributes(-1, _("Name"), gtk.CellRendererText(), markup=2)
		self.viewNewcommands.insert_column_with_attributes(-1, _("Definition"), gtk.CellRendererText(), text=1)
		self.viewNewcommands.connect("row-activated", self.onNewcommandSelected)
		self.viewNewcommands.connect("button-press-event", self.onNewcommandContext)
		
		scrNewcommands = gtk.ScrolledWindow()
		scrNewcommands.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC);
		scrNewcommands.set_shadow_type(gtk.SHADOW_IN)
		scrNewcommands.add(self.viewNewcommands)
		vbox.add(scrNewcommands)
		
		btnAddNewcommand = gtk.Button(_("Add New Command"))
		btnAddNewcommand.set_relief(gtk.RELIEF_NONE)
		btnAddNewcommand.set_image(gtk.image_new_from_stock(gtk.STOCK_ADD, gtk.ICON_SIZE_MENU))
		btnAddNewcommand.connect("clicked", lambda a: self.runMacroDialog(const.ADD_NEWCOMMAND))
		
		vbox.pack_start(btnAddNewcommand, False)
		
		self.append_page(vbox, gtk.Label(_("New Commands")))
	
	def onNewcommandSelected(self, treeview, path, column):
		"""
		Inserts a newcommand into the document
		(called on 'row-activated' of the listview)
		"""
		liststore = treeview.get_model()
		command = liststore.get_value(liststore.get_iter(path), 0)
		self._instance.get_document().insert(command)
	
	def onNewcommandContext(self, treeview, event):
		"""
		Displays the context menu for newcommand entries
		"""
		if event.button == 3:
			x, y = int(event.x), int(event.y)
			pathTuple = treeview.get_path_at_pos(x, y)
            
			if pathTuple is not None:
				path = pathTuple[0]
				treeview.grab_focus()
				treeview.set_cursor(path)
		
			selection = treeview.get_selection()
			if not selection.count_selected_rows():
				return
			self._instance._manager.get_widget("/NewcommandPopup").popup(None, None, None, event.button, event.time)
	
	def onNewcommandContextJump(self, action):
		""" Jumps to the definition of a newcommand """
		liststore, listiter = self.viewNewcommands.get_selection().get_selected()
		#start = liststore.get_value(listiter, 3)
		#end = liststore.get_value(listiter, 4)
		#self._instance.get_document().selectRange(start, end)
		string = liststore.get_value(listiter, 3)
		self._instance.get_document().findString(string)
	
	def onNewcommandContextInsert(self, action):
		"""
		Inserts a newcommand into the document
		(called from the context menu)
		"""
		liststore, listiter = self.viewNewcommands.get_selection().get_selected()
		command = liststore.get_value(listiter, 0)
		self._instance.get_document().insert(command)
	
	def onNewcommandContextEdit(self, action):
		pass
	
	def onNewcommandContextDelete(self, action):
		pass


class MacroDialog(gtk.Dialog):
	""" Dialog for adding/editing a user-defined macro and adding a new command """
	mode = None
	
	def __init__(self, instance):
		import gtksourceview, pango
		from PreviewArea import PreviewArea
	
		gtk.Dialog.__init__(self, None, None, gtk.DIALOG_DESTROY_WITH_PARENT, (gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL, gtk.STOCK_OK, gtk.RESPONSE_OK))
		self.set_border_width(5)
		self.set_has_separator(False)
		self.vbox.set_spacing(10)
		
		# --- name ---
		self.entryName = gtk.Entry()
		self.entryName.show()
		lblName = gtk.Label(_("_Name:"))
		lblName.show()
		lblName.set_use_underline(True)
		lblName.set_alignment(0, .5)
		lblName.set_mnemonic_widget(self.entryName)
		
		# --- definition ---
		manager = gtksourceview.SourceLanguagesManager()
		language = manager.get_language_from_mime_type("text/x-tex")
		self.bufDef = gtksourceview.SourceBuffer()
		self.bufDef.set_language(language)
		self.bufDef.set_highlight(True)
		
		srcDefinition = gtksourceview.SourceView(self.bufDef)
		srcDefinition.show()
		srcDefinition.set_wrap_mode(gtk.WRAP_WORD)
		srcDefinition.modify_font(pango.FontDescription("Monospace"))
		
		scrDefinition = gtk.ScrolledWindow()
		scrDefinition.show()
		scrDefinition.set_policy(gtk.POLICY_NEVER, gtk.POLICY_AUTOMATIC);
		scrDefinition.set_shadow_type(gtk.SHADOW_IN)
		scrDefinition.add(srcDefinition)
		
		lblDef = gtk.Label(_("_Definition:"))
		lblDef.show()
		lblDef.set_use_underline(True)
		lblDef.set_mnemonic_widget(scrDefinition)
		lblDef.set_alignment(0, 0)
		
		# --- packages ---
		self.cmbPackage = gtk.combo_box_entry_new_text()
		self.cmbPackage.show()
		for item in ["array", "amsmath", "amssymb"]:
			self.cmbPackage.append_text(item)
		
		lblPackages = gtk.Label(_("Required _Packages:"))
		lblPackages.show()
		lblPackages.set_use_underline(True)
		lblPackages.set_mnemonic_widget(self.cmbPackage)
		lblPackages.set_alignment(0, .5)
		
		self.storePackages = gtk.ListStore(str)
		self.viewPackages = gtk.TreeView(self.storePackages)
		self.viewPackages.show()
		self.viewPackages.insert_column_with_attributes(-1, _("Name"), gtk.CellRendererText(), text=0)
		self.viewPackages.set_headers_visible(False)
		scrPackages = gtk.ScrolledWindow()
		scrPackages.show()
		scrPackages.set_policy(gtk.POLICY_NEVER, gtk.POLICY_AUTOMATIC)
		scrPackages.set_shadow_type(gtk.SHADOW_IN)
		scrPackages.add(self.viewPackages)
		
		btnAddPackage = gtk.Button()
		btnAddPackage.show()
		btnAddPackage.set_relief(gtk.RELIEF_NONE)
		image = gtk.image_new_from_stock(gtk.STOCK_ADD, gtk.ICON_SIZE_BUTTON)
		image.show()
		btnAddPackage.add(image)
		btnAddPackage.connect("clicked", lambda a: self.addPackage())
		
		btnRemPackage = gtk.Button()
		btnRemPackage.show()
		btnRemPackage.set_relief(gtk.RELIEF_NONE)
		image = gtk.image_new_from_stock(gtk.STOCK_REMOVE, gtk.ICON_SIZE_BUTTON)
		image.show()
		btnRemPackage.add(image)
		btnRemPackage.connect("clicked", lambda a: self.removePackage())
		algRemPackage = gtk.Alignment(.5, 0, 1, 0)
		algRemPackage.show()
		algRemPackage.add(btnRemPackage)
		
		tblPackages = gtk.Table()
		tblPackages.show()
		tblPackages.attach(self.cmbPackage, 0, 1, 0, 1, gtk.FILL|gtk.EXPAND, False)
		tblPackages.attach(btnAddPackage, 1, 2, 0, 1, False, False)
		tblPackages.attach(scrPackages, 0, 1, 1, 2, gtk.FILL|gtk.EXPAND, gtk.FILL|gtk.EXPAND)
		tblPackages.attach(algRemPackage, 1, 2, 1, 2, False, gtk.FILL)
		
		# --- scope ---
		self.cmbScope = gtk.combo_box_new_text()
		self.cmbScope.show()
		for item in [_("General and mathematical"), _("General only"), _("Mathematical only")]:
			self.cmbScope.append_text(item)
		self.cmbScope.set_active(0)
		
		lblScope = gtk.Label(_("_Scope:"))
		lblScope.show()
		lblScope.set_use_underline(True)
		lblScope.set_mnemonic_widget(self.cmbScope)
		lblScope.set_alignment(0, .5)
		
		# --- preview ---
		self.preview = PreviewArea()	
		self.preview.show()
		scrPreview = gtk.ScrolledWindow()
		scrPreview.show()
		scrPreview.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
		scrPreview.set_shadow_type(gtk.SHADOW_NONE)
		scrPreview.add_with_viewport(self.preview)
		scrPreview.connect("button-press-event", lambda *a: self.renderPreview())
		
		lblPreview = gtk.Label()
		lblPreview.show()
		lblPreview.set_markup(_("Preview:\n<small>Click to reload</small>"))
		lblPreview.set_alignment(0, 0)
		
		table = gtk.Table()
		table.show()
		table.set_border_width(5)
		table.set_row_spacings(5)
		table.set_col_spacings(10)
		table.attach(lblName, 0, 1, 0, 1, gtk.FILL, gtk.FILL)
		table.attach(self.entryName, 1, 2, 0, 1, gtk.EXPAND|gtk.FILL, False)
		table.attach(lblDef, 0, 1, 1, 2, gtk.FILL, gtk.FILL)
		table.attach(scrDefinition, 1, 2, 1, 2)
		table.attach(lblPreview, 0, 1, 2, 3, gtk.FILL, gtk.FILL)
		table.attach(scrPreview, 1, 2, 2, 3)
		self.vbox.add(table)
		
		tblAdv = gtk.Table()
		tblAdv.show()
		tblAdv.set_row_spacings(5)
		tblAdv.set_col_spacings(10)
		tblAdv.attach(lblPackages, 0, 1, 0, 1, gtk.FILL, gtk.FILL)
		tblAdv.attach(tblPackages, 1, 2, 0, 1)
		tblAdv.attach(lblScope, 0, 1, 1, 2, gtk.FILL, False)
		tblAdv.attach(self.cmbScope, 1, 2, 1, 2, gtk.EXPAND|gtk.FILL, gtk.FILL)
		
		algAdv = gtk.Alignment(.5, .5, 1, 1)
		algAdv.show()
		algAdv.set_padding(0, 0, 20, 5)
		algAdv.add(tblAdv)
		
		self.expander = gtk.Expander(_("<b>_Advanced options</b>"))
		self.expander.set_use_markup(True)
		self.expander.set_use_underline(True)
		self.expander.add(algAdv)
		self.vbox.pack_start(self.expander, False)
		
		# store a pointer to the OKAY button
		for child in self.action_area.get_children():
			if self.get_response_for_widget(child) == gtk.RESPONSE_OK:
				child.set_sensitive(False)
				self._button_okay = child
		self.entryName.connect("changed", self.onNameChanged)
		self.bufDef.connect("changed", self.onDefChanged)
		
		srcDefinition.set_size_request(300, 70)
		
		self.connect("response", self.onResponse)
		self.set_default_response(gtk.RESPONSE_OK)
	
	def addPackage(self):
		self.storePackages.append([self.cmbPackage.get_active_text()])
	
	def removePackage(self):
		listStore, treeIter = self.viewPackages.get_selection().get_selected()
		try:
			listStore.remove(treeIter)
		except TypeError:
			pass
	
	def loadMacro(self, name, definition, scope, packages):
		self.set_title(_("Edit Macro - %s") % name)
		self.entryName.set_text(name)
		self.bufDef.set_text(definition)
		self.cmbScope.set_active(scope)
		self.storePackages.clear()
		for package in packages:
			self.storePackages.append([package])
		self.renderPreview()
	
	def run(self, mode):
		self.mode = mode
		self.set_title({const.ADD_MACRO:_("Add Macro"),const.EDIT_MACRO:_("Edit Macro"),const.ADD_NEWCOMMAND:_("Add New Command")}[mode])
		if mode == const.ADD_NEWCOMMAND:
			self.expander.hide()
			self.reset()
		else:
			self.expander.show()
			if mode == const.ADD_MACRO:
				self.reset()
		return gtk.Dialog.run(self)
	
	def reset(self):
		self.entryName.set_text("")
		self.bufDef.set_text("")
		#self.cmbPackage.set_active_text("")
		self.storePackages.clear()
		self.preview.clear()
		self.cmbScope.set_active(0)
	
	def renderPreview(self):
		source = self.bufDef.get_text(self.bufDef.get_start_iter(), self.bufDef.get_end_iter())
		if self.mode == const.ADD_NEWCOMMAND:
			self.preview.setEscapeRefs(True)
		else:
			if self.cmbScope.get_active() == 2:
				source = "$%s$" % source
		self.preview.setPackages([row[0] for row in self.storePackages])
		self.preview.setSource(source)
		self.preview.render()
	
	def onNameChanged(self, entry):
		if bool(len(self.bufDef.get_text(self.bufDef.get_start_iter(), self.bufDef.get_end_iter()))):
			self._button_okay.set_sensitive(bool(len(self.entryName.get_text())))
	
	def onDefChanged(self, textbuffer):
		if bool(len(self.entryName.get_text())):
			self._button_okay.set_sensitive(bool(len(textbuffer.get_text(textbuffer.get_start_iter(), textbuffer.get_end_iter()))))
	
	def onResponse(self, dialog, response):
		if response == gtk.RESPONSE_OK:
			self.macroName = self.entryName.get_text()
			self.definition = self.bufDef.get_text(self.bufDef.get_start_iter(), self.bufDef.get_end_iter())
			self.scope = self.cmbScope.get_active()
			self.packages = [r[0] for r in self.storePackages]
